DISASM .ORG $B000 ; DISASSEMBLER FOR THE MCS-6502
; VERSION 0 March 1982 Hans Otten
; Disassembly subroutine by Wozniak/Baum 
;
;   ORIGINAL SOURCE CODE IN MICRO ADE FORMAT FROM 
;   http://retro.hansotten.nl/6502-sbc/kim-1-manuals-and-software/kim-1-software/#pdis
;
; VERSION 1.1 JANUARY 2023  SYNTAX MODIFIED FOR THE
;  A65 CROSS ASSEMBLER   GLEN DEAS, K5GED  USA
;    INVOKE A65:  A65 -L -M -H D6502TTYV2.S
; Version 2.0  MODIFIED TO USE THE KIM-1 SERIAL I/O INSTEAD OF PARALLEL KEYBOARD
;               USING KIM'S TTY ROUTINES - BY DEFAULT IT OUTPUTS TO THE VIDEO 
;               MONITOR/TTY VIA THE OUTCH CALL TO THE KIM-1 MONITOR
; 
; V3 Hans Otten, January 17, 2023 KIM-1 TTY version
; 
; changed to TASM32 syntax 
;  NPPEXEC script 
;  tasm -65 -x3 -g1 -s "PRDISV3.ASM" "PRDISV3.pap"  "PRDISV3.lst"  -s "PRDISV3.sym"
;
; Changed to paginated or simple output format, traces of H14 serial printer and parallel keyboard present in comments
; 8 bit character display 
; print/video behaviour changed
;
; How to use
;
; Load and start program at $B000, Uses buffer at $0200
; Choose paginated or not, enter P or P for paginated 
; enter start and address in hex, allowed is  
;   0100 0300
;   0100-0300
;   0100,0300 
; backspace works
; Enter header text if paginated
; Disassembly starts now
; if not paginated output stops after page, ENter to continue., other key stops
; at end monitor is entered at B000
; 
; MIT License
;
;  Copyright (c) 2021 Hans Otten 
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;
; ZERO PAGE DEFINES
;
PRTFLG 	= $00EE 		; PRINTER FLAG
FORMAT 	= $00EF ;
LENGTH 	= FORMAT+1
LMNEM 	= LENGTH+1
RMNEM 	= LMNEM+1
COUNT 	= RMNEM+3
YSAVE 	= COUNT+1
PCL 	= YSAVE+4
PCH 	= PCL+1
STRPTL 	= $00EA 		; message string pointer 
STRPTH 	= STRPTL + 1 		
TEMP 	= $00FC 
CHAR 	= $00EF 
;
; PIA ADDRESSES (not used, for parallel keyboard and H14 printer)
;
;PIAAD 	= $1700		; PIA A DATA 
;PIAADD = $1701 	; PIA A DATA DIR
;PIABD 	= $1702		; PIA B DATA
;PIABDD = $1703 	; PIA B DATA DIR
;
;     SUBROUTINES IN KIM-1 monitor
;
MONIT = $1C4F 	;
OUTCHK = $1EA0 	;  SERIAL DATA OUT
GETCHK = $1E5A  ;  SERIAL DATA IN (with hardware echo)
;
; CONSTANTS
;
FF 		= $0C ; Formfeed
BL 		= $20 ; space
CR 		= $0D ; Carriage Return
LF 		= $0A ; Linefeed
BS 		= $08 ;
CANCEL 	= $18 ;
ESC		= $1B ;
;
;
; variables (should be in RAM, here $0200)
;
PAGECT 	= $0200			; PAGE COUNTER
LINECT 	= PAGECT + 1 	; LINE COUNTER
BEGADL 	= LINECT + 1 	; BEGINADRES
BEGADH  = BEGADL + 1	;
POINTL  = BEGADH + 1 	; ADDRESS POINTER
POINTH  = POINTL + 1	;
BUFFER  = POINTH + 1 	; INPUT BUFFER 64 CHARACTER
;
; MAIN PROGRAM INITITIALISTAION
;
START 	CLD  				; just in case (page numbers calculated in decimal mode!)
		LDX #$FF 			; INIT STACK
		TXS  ;
		LDA #$01 			; FIRST PAGE
		STA PAGECT ;
		LDA #$00 ;
		STA PRTFLG ;
		STA LINECT 			; LINE := 0
		JSR CRLFOUT ;
		LDX #(GRTMSG >> 8) 	; BEG. LOW ADDR BYTE POINTER PRINT 'DISASSEMBLER 6502'
		LDY #(GRTMSG & $FF) ; LOCATION HIGH BYTE  
		JSR PRTSTR ;
		JSR CRLFOUT ;
		LDX #(PRTMSG >> 8) 	; PRINT 'PRINTER OR VIDEO (P/V) ?'
		LDY #(PRTMSG & $FF)
		JSR PRTSTR ;
		JSR BUFIN 			;  ASK FOR INPUT
		LDX #$00 
		LDA BUFFER,X 
		CMP #'P' 			; ANSWER IS 'P' ?
		BEQ DOPRT
 		CMP #'p' 			; ANSWER IS 'P' ?
		BNE NOPRT 
DOPRT 	LDA #$01 
		STA PRTFLG 
NOPRT 	JSR CRLFOUT 
ADR 	LDX #(STADRM >> 8)	; PRINT STARTADDRES
		LDY #(STADRM & $FF) 
		JSR PRTSTR 
		JSR BUFIN 
		LDX #$00 
		JSR GETADR 
		LDA POINTH 
		STA BEGADH 
		LDA POINTL 
		STA BEGADL 
NEXTCH 	LDA BUFFER,X 		;
		CMP #' '  			; BLANK IS DELIMITER
		BEQ SKIP 			;
		CMP #',' 			;
		BEQ SKIP 			; ',' IS DELIMITER
		CMP #'-' 			; '-' IS DELIMITER
		BNE ENDA 			; MAYBE NUMBER
SKIP 	INX  				; SKIP DELIMITER
		JMP NEXTCH 			;
ENDA 	JSR GETADR 			;
		CPX #$02 			;
		BCC NOPRT 			; NOT RIGHT INPUT
		LDA BEGADH 			; IS BEGINADR < ENDADR ?
		CMP POINTH 			;
		BCC OK 				;
		BEQ LOWCH 			;
		BNE NOPRT 			; NO TRY AGAIN
LOWCH 	LDA POINTL 			;
		CMP BEGADL 			;
		BCC NOPRT 			;
;
; GET HEADER
;
OK 		LDA PRTFLG 			; IF PRINTER THEN ASK FOR HEADER
		BNE HDIS ;
;		
		LDX #(STMSG >> 8) 	;
		LDY #(STMSG & $FF) 	; else print 'Start disassembly on video
		JSR PRTSTR ;
		JSR CRLFOUT 		;
		JMP STDIS
;
; ask for header
;		
HDIS	LDX #(HEAMSG >> 8) 	;
		LDY #(HEAMSG & $FF) ; PRINT 'HEADER'
		JSR PRTSTR 			;
		JSR BUFIN 			;
		LDX #$FF 			;
NEXTB 	INX  				;
		LDA BUFFER,X 		;
		CMP #CR 			; SCAN BUFFER TO CR
		BNE NEXTB 			;
		LDA #BL 			; REPLACE WITH BLANK
NEXTBL 	STA BUFFER,X 		;
		INX  				;
		CPX #$3F 			;
		BNE NEXTBL 			;
		LDA #$A0 			; MAKE LAST END OF STRING
		STA BUFFER,X 		;
;
; START DISASSEMBLY
;
 
STDIS		LDA BEGADL 			;
		STA PCL 			;
		LDA BEGADH 			;
		STA PCH ;
FURDER 	JSR INSTDS 			; DISASSEMBLE INSTRUCTION
		JSR PCADJ 			; NEXT ADDRESS
		STA PCL ;
		STY PCH ;
		CPY POINTH 			;
		BNE FURDER 			;
		CMP POINTL 			;
		BCC FURDER 			;
		JSR CRLFOUT 		;
		LDA #FF				; formfeed on printer
		JSR OUTCH 			;
STOP						; escape stop
		LDA #$00 ;
		STA PRTFLG 			;
		LDA #(START >> 8) 	; SET UP START ADDRESS
		STA PCH ;
		LDA #(START & $FF) 	;
		STA PCL ;
		JMP MONIT 			; BACK TO MONITOR
   ;
   ; SUBROUTINES FOLLOW :
   ;     DISASSEMBLE ONE INSTRUCTION (wozniak/Baum)
   ;
INSTDS 	JSR PRPC 		; PRINT PCL PCH
		LDA (PCL,X) 	; GET OPCODE
		TAY  ;
		LSR A  			; EVEN/ODD TEST
		BCC IEVEN 		;
		LSR A  			; TEST B1
		BCS ERR 		; XXXXXX11 INSTRUCTION INVALID
		CMP #$22 		;	
		BEQ ERR 		; 10001001 INSTRUCTION INVALID
		AND #$07 		; MASK 3 BITS FOR ADDR MODE
		ORA #$80 		; ADD INDEXING OFFSET
IEVEN 	LSR A  			; LSB INTO CARRY FOR
		TAX  			; LEFT/RIGHT TEST BELOW
		LDA MODE,X 		; INDEX INTO ADDRESSING MODE TABLE
		BCS RTMODE 		; IF CARRY SET USE LSD FOR
		LSR A  			; PRINT FORMAT INDEX
		LSR A  			;
		LSR A  			; IF CARRY CLEAR USE MSD
		LSR A  			;
RTMODE 	AND #$0F 		; MASK FOR 4-BIT INDEX
		BNE GETFMT 		; 0 FOR INVALID OPCODES
ERR 	LDY #$80 		; SUBSTITUTE $80 FOR INVALID OPCD.
		LDA #$00 		; SET PRINT FORMAT INDEX TO 0
GETFMT 	TAX  ;
		LDA MODEB,X 	; INDEX INTO PRINT FORMAT TABLE
		STA FORMAT 		; SAVE FOR ADDR.FIELD FORMAT
		AND #$03 		; MASK 2-BIT LENGTH. 0=1-BYTE
		STA LENGTH 		; 1=2 BYTE, 2=3 BYTE
		TYA  			; OPCODE
		AND #$8F 		; MASK IT FOR 1XXX1010 TEST
		TAX  			; SAVE IT
		TYA  			; OPCODE AGAIN TO A
		LDY #$03 		;
		CPX #$8A 		;	
		BEQ MNNDXC 		;
MNNDXA 	LSR A  			;
		BCC MNNDXC 		; FORM INDEX INTO MNEMONIC TABLE
		LSR A  			;
MNNDXB 	LSR A  			; 1XXX1010 => 00101XXX
		ORA #$20 		; XXXYYY01 => 00111XXX
		DEY  			; XXXYYY10 => 00110XXX
		BNE MNNDXB 		; XXXYY100 => 00100XXX
		INY  			; XXXXX000 => 000XXXXX
MNNDXC 	DEY  			;
		BNE MNNDXA 		;
		PHA  			; SAVE MNEMONIC TABLE INDEX
PROP 	LDA (PCL),Y 	;
		JSR PRBYTE 		;
		LDX #$01 		;
PROPBL 	JSR PRBLB 		;
		CPY LENGTH 		; PRINT INSTR (1 TO 3 BYTES)
		INY  			; IN A 12-CHARACTER FIELD
		BCC PROP 		;
		LDX #$03 		; CHAR COUNT FOR PRINTING MNEMONIC
		CPY #$04 		;
		BCC PROPBL 		;
		PLA  			; RECOVER MNEMONIC INDEX
		TAY  			;
		LDA MNEML,Y 	;
		STA LMNEM 		; FETCH 3-CHAR MNEMONIC
		LDA MNEMR,Y 	; PACKED IN 2 BYTES
		STA RMNEM 		;
PRMNA 	LDA #$00 		;
		LDY #$05 		;
PRMNB 	ASL RMNEM 		;
		ROL LMNEM 		; SHIFT 5 BITS OF CHAR IN ACCU
		ROL A  			; CLEARS CARRY
		DEY  			;
		BNE PRMNB 		;
		ADC #$BF 		; ADD 'X' OFFSET
		JSR OUTCH 		;  OUTPUT A CHAR OF MNEMONIC
		DEX  ;
		BNE PRMNA 		;
		JSR PRBLNK 		; OUTPUT 1 BLANK
		LDX #$06 		; COUNT FOR 6 PRINT-FORMAT BITS
PRADRA 	CPX #$03 		;
		BNE PRADRC 		; IF X=3 PRINT ADD VALUE
		LDY LENGTH 		;
		BEQ PRADRC 		; NO PRINT IF LENGTH = 0
PRADRB 	LDA FORMAT 		;
		CMP #$E8 		; HANDLE REL ADDRESSING MODE
		LDA (PCL),Y 	; SPECIAL (PRINT TARGET ADDRESS)
		BCS RELADR 		; (NOT DISPLACEMENT)
		JSR PRBYTE 		; OUTPUT 1- OR 2-BYTE ADDRESS
		DEY  			; MOST SIGNIFICANT BYTE FIRST
		BNE PRADRB 		;
PRADRC 	ASL FORMAT 		; TEST NEXT PRINT FORMAT BIT
		BCC PRADRD 		; IF 0, DON'T PRINT
		LDA CHARA-1,X 	; CORRESPONDING CHARACTER
		JSR OUTCH 		;  OUTPUT 1 OR 2 CHARACTERS
		LDA CHARB-01,X 	; (IF CHAR FROM CHARB IS 0)
		BEQ PRADRD 		; DON'T OUTPUT IT
		JSR OUTCH 		;
PRADRD 	DEX  			;
		BNE PRADRA 		;
		RTS  			; RETURN IF 6 FORMAT BITS
;
RELADR 	JSR PCADJB 		; PCL, PCH + DISPL + 1 TO A,Y
		TAX  			;
		INX  			;
		BNE PRNTYX 		; +1 TO X,Y
		INY  			;
PRNTYX 	TYA  			;
PRNTAX 	JSR PRBYTE 		; PRINT TARGET ADDRESS OF BRANCH
PRNTX 	TXA  			; AND RETURN
		JMP PRBYTE 		;
PRPC 	LDA #$8D 		;
		JSR LFCR 		; OUTPUT CARRIAGE RETURN
		LDA PCH 		;
		LDX PCL 		;
		JSR PRNTAX 		; OUTPUT PCH AND PCL
		LDA #$20 		;
		JSR OUTCH 		;
PRBLNK 	LDX #$01 		; BLANK COUNT
PRBLB 	LDA #' '  		;	
PRBLC 	JSR OUTCH 		;  OUTPUT A BLANK
		DEX  			;
		BNE PRBLB 		; LOOP UNTIL COUNT IS ZERO
		RTS  			;
;
PCADJ 	LDA LENGTH 		; 0=1 BYTE, 1=2 BYTE, 2=3 BYTE
PCADJB 	SEC  			;
PCADJC 	LDY PCH 		;
		TAX  			; TEST DISPLACEMENT SIGN FOR REL.
		BPL PCADJD 		; ADDRESSING MODE
		DEY  			; EXTEND NEG BY DECREMENTING PCH
PCADJD 	ADC PCL 		;
		BCC RTSA 		; PCL+LENGTH (OR DISPLACEMENT) +1 
		INY  			;
RTSA 	RTS  			;
   ;
   ; DISASSEMBLER TABLES
   ;
MODE .BYTE $40 ;
 .BYTE $02
 .BYTE $45
 .BYTE $03
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $30 ; XXXXXXZ0 INSTRUCTIONS
 .BYTE $22
 .BYTE $45 ; Z=0, LEFT  NIBBLE
 .BYTE $33 ; Z=1, RIGHT NIBBLE
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $40
 .BYTE $02
 .BYTE $45
 .BYTE $33
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $40
 .BYTE $02
 .BYTE $45
 .BYTE $B3
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $00
 .BYTE $22
 .BYTE $44
 .BYTE $33
 .BYTE $D0
 .BYTE $8C
 .BYTE $44
 .BYTE $00
 .BYTE $11
 .BYTE $22
 .BYTE $44
 .BYTE $33
 .BYTE $D0
 .BYTE $8C
 .BYTE $44
 .BYTE $9A
 .BYTE $10
 .BYTE $22
 .BYTE $44
 .BYTE $33
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $10
 .BYTE $22
 .BYTE $44
 .BYTE $33
 .BYTE $D0
 .BYTE $08
 .BYTE $40
 .BYTE $09
 .BYTE $62
 .BYTE $13 ; YYXXXZ01 INSTRUCTIONS
 .BYTE $78
 .BYTE $A9
MODEB .BYTE $00 ; ERROR
 .BYTE $21 ; IMMEDIATE
 .BYTE $81 ; ZERO PAGE
 .BYTE $82 ; ABSOLUTE
 .BYTE $00 ; IMPLIED
 .BYTE $00 ; ACCUMULATOR
 .BYTE $59 ; INDEXED INDIRECT
 .BYTE $4D ; INDIRECT INDEXED
 .BYTE $91 ; ZEROPAGE INDEXED BY REGISTER X
 .BYTE $92 ; ABSOLUTE INDEXED BY REGISTER X
 .BYTE $86 ; ABSOLUTE INDEXED BY REGISTER Y
 .BYTE $4A ; ABSOLUTE
 .BYTE $85 ; ZEROPAGE INDEXED BY REGISTER Y
 .BYTE $9D ; RELOCATABLE
CHARA .BYTE  ','
 .BYTE  ')'
 .BYTE  ','
 .BYTE  '#'
 .BYTE  '('
 .BYTE  '$'
CHARB .BYTE  'Y'
 .BYTE $00
 .BYTE  'X'
 .BYTE  '$'
 .BYTE  '$'
 .BYTE $00
MNEML .BYTE $1C ; XXXXX000 INSTRUCTIONS
 .BYTE $8A
 .BYTE $1C
 .BYTE $23
 .BYTE $5D
 .BYTE $8B
 .BYTE $1B
 .BYTE $A1
 .BYTE $9D
 .BYTE $8A
 .BYTE $1D
 .BYTE $23
 .BYTE $9D
 .BYTE $8B
 .BYTE $1D
 .BYTE $A1
 .BYTE $00
 .BYTE $29
 .BYTE $19
 .BYTE $AE
 .BYTE $69
 .BYTE $A8
 .BYTE $19
 .BYTE $23
 .BYTE $24
 .BYTE $53
 .BYTE $1B
 .BYTE $23
 .BYTE $24
 .BYTE $53
 .BYTE $19
 .BYTE $A1
 .BYTE $00 ; XXXYY100 INSTRUCTIONS
 .BYTE $1A
 .BYTE $5B
 .BYTE $5B
 .BYTE $A5
 .BYTE $69
 .BYTE $24
 .BYTE $24
 .BYTE $AE ; 1XXX1010 INSTRUCTIONS
 .BYTE $AE
 .BYTE $A8
 .BYTE $AD
 .BYTE $29
 .BYTE $00
 .BYTE $7C
 .BYTE $00
 .BYTE $15 ; XXXYYY10 INSTRUCTIONS
 .BYTE $9C
 .BYTE $6D
 .BYTE $9C
 .BYTE $A5
 .BYTE $69
 .BYTE $29
 .BYTE $53
 .BYTE $84 ; XXXYYY01 INSTRUCTIONS
 .BYTE $13
 .BYTE $34
 .BYTE $11
 .BYTE $A5
 .BYTE $69
 .BYTE $23
 .BYTE $A0
MNEMR .BYTE $D8 ; XXXXX000 INSTRUCTIONS
 .BYTE $62
 .BYTE $5A
 .BYTE $48
 .BYTE $26
 .BYTE $62
 .BYTE $94
 .BYTE $88
 .BYTE $54
 .BYTE $44
 .BYTE $C8
 .BYTE $54 
 .BYTE $68
 .BYTE $44
 .BYTE $E8
 .BYTE $94 
 .BYTE $00
 .BYTE $B4
 .BYTE $08
 .BYTE $84
 .BYTE $74
 .BYTE $B4
 .BYTE $28
 .BYTE $6E
 .BYTE $74
 .BYTE $F4
 .BYTE $CC
 .BYTE $4A
 .BYTE $72
 .BYTE $F2
 .BYTE $A4
 .BYTE $8A
 .BYTE $00 ; XXXYY100 INSTRUCTIONS
 .BYTE $AA
 .BYTE $A2
 .BYTE $A2
 .BYTE $74
 .BYTE $74
 .BYTE $74
 .BYTE $72
 .BYTE $44 ; 1XXX1010 INSTRUCTIONS
 .BYTE $68
 .BYTE $B2
 .BYTE $32
 .BYTE $B2
 .BYTE $00
 .BYTE $22
 .BYTE $00
 .BYTE $1A ; XXXYYY10 INSTRUCTIONS
 .BYTE $1A
 .BYTE $26
 .BYTE $26
 .BYTE $72
 .BYTE $72
 .BYTE $88
 .BYTE $C8
 .BYTE $C4 ; XXXYYY01 ISTRUCTIONS
 .BYTE $CA
 .BYTE $26
 .BYTE $48
 .BYTE $44
 .BYTE $44
 .BYTE $A2
 .BYTE $C8
   ;
   ; LFCR ROUTINE FOR DISASSEMBLY, CONTROL LAYOUT
   ;
LFCR 	LDA PRTFLG 
		BEQ VID 
		LDA LINECT 
		BNE NOH 			; IF LINECT = 0 THEN
		JSR PRINTH 			;   PRINT HEADER
		SED  					; 
		LDA PAGECT 			;
		CLC  ;
		ADC #$01 			; PAGECT := PAGECT + 1
		STA PAGECT 			;
		CLD  ;
NOH 	INC LINECT 			;
		LDA LINECT 			; IF LINECT = 55 THEN
		CMP #$37 			;   LINECT := 0
		BNE NOTEND			;
		LDA #$00 			;
		STA LINECT 			;
NOTEND 	JMP DOCRLF	 		;	
VID 	LDA LINECT 			; ELSE
		CMP #$14 			;   IF LINECT = 20 THEN
		BNE NOPAGE 			;
		JSR INKEYB 			;  GETKEY
		CMP #CR 			;   IF KEY = CR THEN
		BEQ NOSTOP			;   ELSE if key = ESCAPE then stop 
		JMP STOP
		LDA POINTL 			; BEGIN ADR := END ADDRESS
		STA PCL 			;
		LDA POINTH 			;
		STA PCH 			;
NOSTOP 	LDA #$01 			;
		STA LINECT 			;
		JMP DOCRLF 			;
NOPAGE 	INC LINECT 			;
DOCRLF 	JSR CRLFOUT 		;
		RTS  				;
   ;
   ; PRINT HEADER
   ;
PRINTH 	JSR CRLFOUT 			;
		JSR CRLFOUT 			;
		LDA #FF 				; Form feed on printer only
		JSR OUTCH 					;
		LDX #(BUFFER >> 8) 		; PRINT HEADER FROM BUFFER
		LDY #(BUFFER & $FF) 	;
		JSR PRTSTR 				;
		LDX #(PAGMSG >> 8) 		; PRINT 'PAGE'
		LDY #(PAGMSG & $FF) 	;
		JSR PRTSTR 				;
		LDA PAGECT 				;
		JSR PRBYTE 				; pagenr in decimal
		JSR CRLFOUT 			;
		RTS  ;
 ;
 ; CARRIAGE RETURN LINEFEED
 ;
CRLFOUT LDA #CR 				;
		JSR OUTCH 				; Carriage return
		LDA #LF 				;
		JSR OUTCH 				; Linefeed
		RTS  					;
;
; PRINT BYTE IN A AS TWO ASCII
 
PRBYTE 	STY YSAVE 				;
		STA TEMP 				;
		LSR A  					;
		LSR A  					; LEFT NIBBLE FIRST
		LSR A  					;
		LSR A  					;
		JSR HEXTA 				;
		LDA TEMP 				;
		JSR HEXTA 				;	
		LDA TEMP 				;
		LDY YSAVE 				;
		RTS  					;
;
; print low byte nibble value as hex character
  
HEXTA 	AND #$0F 				;
		CMP #$0A 				;
		CLC  					;
		BMI HEXTAI 				;
		ADC #$07 				;
HEXTAI 	ADC #$30 				;
		JMP OUTCH 				;
   ;
   ;
    ;PRINTER OUTPUT H14 WITH HANDSHAKE
   ;
   ; does not work if printer not present, s RTS for now
OUTPR		RTS ;
 ;		PHP  ; SAVE REGISTERS ON STACK
 ;		PHA  ;
 ;		TYA  ;
 ;		PHA  ;
 ;		TXA  ;
 ;		PHA  ;
 ;		LDA CHAR ;
 ;		PHA  ;
 ;		TSX  ;
 ;		LDA $0104,X ;
 ;		STA CHAR ;
;BUFFV 	LDA PIABD ; PRINTER RTS
; 		AND #$02 ; IN BIT 2
; 		BEQ BUFFV ; PRINTER READY ?
; 		LDA PIABD ;
; 		ORA #$01 ; STARTBIT
; 		STA PIABD ;
; 		JSR TDELAY ;
; 		LDX #$08 ; 8 BITS TO SEND 
; 		BPL VOLGB ; DUMMY
;VOLGB 	LDA PIABD ;
; 		LSR CHAR ; LSB IN CARRY
; 		BCS EEN ;
; 		ORA #$01 ; BIT IS 0
; 		BCC BUIT ;
; 		NOP  ; TIMING
;EEN 	AND #$FE ; BIT IS 1
;BUIT 	STA PIABD ;
;		JSR TDELAY ;
;		DEX  ; NEXT BIT
; 		BNE VOLGB ;
; 		LDA PIABD ;
;		AND #$FE ; STOPBIT
; 		NOP  ;
; 		NOP  ;
; 		NOP  ; TIMING
; 		BPL DUMM ;
;DUMM 	STA PIABD ;
; 		JSR TDELAY ;
; 		PLA  ;
; 		STA CHAR ;
; 		PLA  ;
; 		TAX  ;
; 		PLA  ;
; 		TAY  ;
; 		PLA  ;
; 		PLP  ;
; 		RTS  ;
   ;
   ;DELAY ROUTINE 4800 BAUD
   ;
;TDELAY LDY #$22 ;
;DLOOP 	DEY  ;
; 		BNE DLOOP ;
; 		NOP  ;
; 		RTS  ;
   ;
   ; INIT PRINTER = 96 CHARACTERS
   ;
INITPR  RTS
;		LDA PIABDD 		;
;		ORA #$01		;
;		AND #$7D 		;
;		STA PIABDD 		; PORT PRINTER
;		LDA #$00 		; DUMMY START CHARACTER
;		JSR OUTPR 		;
;		LDA #$1B 		; ESCAPE
;		JSR OUTPR 		;
;		LDA #$75 		; 96 CHARACTERS SEQUENCE
;		JSR OUTPR 		;
;		LDA #$10 		;
;		JSR OUTPR 		;
;		RTS  ;
  ;
;======================================
; COMMENTED OUT FOR SERIAL TTY INPUT
;======================================
; PARALLEL INPUT KEYBOARD
;
;INKEYB BIT PIAAD ; STROBE IS BIT 7 
;       BPL INKEYB ;
;STROBE BIT PIAAD ; END OF STROBE ?
;       BMI STROBE ;
;       LDA PIAAD ;
;       RTS  ;
;======================================
;
;========================================
;   SERIAL INPUT FROM TTY-MUST PRESERVE Y
;   THE KIM ROUTINE PRESERVES X, DESTROYS Y
;   AND RETURNS THE CHAR IN A. 
;========================================
INKEYB  TYA 			; MOVE TO ACCUM FOR SAFETY
		STA YSAVE 		; SAVE IT	
		JSR GETCHK 		; READ ONE CHAR FROM SERIAL KB
		PHA 			;  SAVE ACCUM
		LDA YSAVE 		; GET Y BACK
		TAY				; RESTORE Y REG
		PLA 			; GET TTY CHAR BACK
		RTS ;
 ;
 ; PRINT CHARACTER ON VIDEO
 ;
OUTVID 	PHA  			; PUSH REGISTERS ON STACK
		TYA  			;
		PHA  			;
		TXA  			;
		PHA  			;
		TSX  			;
		LDA $0103,X 	; RESTORE ACCUMULATOR
		CMP #FF 		;
		BEQ NOFF 		; REMOVE FORM FEED FROM VIDEO
		AND #$7F
		JSR OUTCHK 		; PRINT ACCU
NOFF 	PLA  			;
		TAX  			;
		PLA  			;
		TAY  			;
		PLA  			; RESTORE REGISTERS
		RTS  			;
;
; PROCEDURE OUT CHARACTER to printer or video
; current no printer, OUTPR is a RTS
;
OUTCH 	PHA  ;
		LDA PRTFLG 		; IF PRINTER THEN
		BEQ NOPNT 		;   OUTPRINTER
        PLA  			;   OUT VIDEO 
        JSR OUTPR 		;
		JSR OUTVID 		;
		RTS				; 
NOPNT 	PLA  			; ELSE	
		JSR OUTVID ;    ;	outvideo 
		RTS  			;
 ;
 ; OUTSTRING
 ;
PRTSTR 	STX STRPTH		; setup string pointer
		STY STRPTL
PRNEXT	LDY #$00 		; REPEAT
		LDA (STRPTL),Y 	;  PRINT(MEM^(STRINGPTR))
		PHA  		 	;  STRINGPTR := STRINGPTR + 1
		JSR OUTCH 		; UNTIL END OF STRING
		PLA  ;
		BMI LAST 		;
		INC STRPTL 		; bit 7 set + end of string
		BNE NEXTS 	
		INC STRPTH 	
NEXTS 	JMP PRNEXT 
LAST 	RTS  		;
   ;
   ; GET LINE IN BUFFER
   ;
BUFIN 	LDY #$00 		; BUFFER POINTER := 0
INB 	JSR INKEYB 		; GET CHARACTER INPUT
		CMP #' '		; CONTROL CHARACTER ?
		BCC CONTROL 	;
		STA BUFFER,Y 	; STORE CHARACTER
		INY  			;
		CPY #$3F 		; NOT MORE THEN 64 CHARACTERS
		BMI INB 		; GO ON
		DEY  			; 
		JMP INB 		; NO ACTION
CONTROL CMP #BS 		; BACKSPACE ?
		BNE DELETE 		;
		JSR OUTCH 		; ECHO BACKSPACE
		LDA #BL 		; 
		JSR OUTCH 		;
		LDA #BS 		;
		JSR OUTCH 		;
		DEY  			; ONE PLACE BACK
NEXTY 	BPL INB 		;
		INY  			; TOO FAR BACK
		JMP NEXTY 		;
DELETE 	CMP #CANCEL 	; CANCEL ?
		BNE EOLN 		;
		JSR CRLFOUT 	;
		JMP BUFIN 		;
		JSR CRLFOUT 	;
EOLN 	CMP #CR 		; END OF LINE ?
		BNE INB 		;
		STA BUFFER,Y 	;
		JSR CRLFOUT 	;
		RTS  			;
   ;
   ; PACK ASCII TO HEX
   ;
ISTHEX 	CMP #$30 		; IF A GE 30 AND A LT 3A THEN
		BCC RET 		;
		CMP #$3A 		;
		BCS LETTER 		;
		BCC SHIFT 		;
LETTER 	CMP #$41 		; IF A GE $41 AND A LT 47 THEN
		BCC RET 		;
		CMP #$47 		;
		BCS RET 		;
		ADC #$09 		;
SHIFT 	AND #$0F 		;
		LDY #$00 		;
		RTS  			;
RET 	LDY #$FF 		; NOT HEX, SET Z
 RTS  ;
   ;
   ; GET ADDRESS  FROM BUFFER,X
   ;
GETADR 	LDA #$00 ;
		STA POINTL 		; CLEAR ADDRESS
		STA POINTH 		;
NEXTAD 	LDA BUFFER,X 	;
		JSR ISTHEX 		; CHECK IF HEX
		BNE GOTADR 		; ELSE READY
		LDY #$04 		; SHIFT ADDRESS 4 BITS LEFT
SHL 	ASL POINTL 		;
		ROL POINTH 		;
		DEY  ;
		BNE SHL ;
		ORA POINTL 		; PLACE LS NIBBLE IN ADDRESS
		STA POINTL 		;
		INX  			;
		JMP NEXTAD ;
GOTADR 	RTS  ;
   ;
   ; TABLES
   ;
GRTMSG 	.TEXT Printing Disassembler V3.0 for 6502
		.BYTE ' ' + $80
   ;
PRTMSG 	.TEXT Printer or video format (p/V)? 
		.BYTE ' '  + $80
   ;
STADRM 	.text Address range (XXXX-YYYY)?
		.BYTE ' ' + $80
   ;
PAGMSG 	.TEXT Page
		.BYTE ' ' + $80
   ;
HEAMSG 	.TEXT Header text?
		.BYTE ' ' + $80
;
STMSG	.TEXT Disassembly:
		.byte ' ' + $80
 .END